# -*- coding: utf-8 -*-

from sfeprapy.func.mcs_gen import _test_dict_flatten as test_mcs_gen_dict_flatten
from sfeprapy.func.mcs_gen import (
    _test_random_variable_generator as test_mcs_gen_random_variable_generator,
)

test_mcs_gen_dict_flatten()
test_mcs_gen_random_variable_generator()
